import { isFn, KeyCode } from '@designable/shared';
export { KeyCode };
var Shortcut = (function () {
    function Shortcut(props) {
        this.codes = props.codes || [];
        this.handler = props.handler;
        this.matcher = props.matcher;
    }
    Shortcut.prototype.preventCodes = function (codes) {
        if (this.codes.length) {
            for (var i = 0; i < codes.length; i++) {
                if (!Shortcut.matchCode(codes[i], this.codes[i])) {
                    return false;
                }
            }
            return true;
        }
        return false;
    };
    Shortcut.prototype.matched = function (matched, context) {
        if (isFn(this.handler) && matched) {
            this.handler(context);
        }
        return matched;
    };
    Shortcut.prototype.match = function (codes, context) {
        var sortedSelf = Shortcut.sortCodes(this.codes);
        var sortedTarget = Shortcut.sortCodes(codes);
        if (isFn(this.matcher)) {
            return this.matched(this.matcher(sortedTarget), context);
        }
        if (sortedTarget.length !== sortedSelf.length)
            return this.matched(false, context);
        for (var i = 0; i < sortedSelf.length; i++) {
            if (!Shortcut.matchCode(sortedTarget[i], sortedSelf[i])) {
                return this.matched(false, context);
            }
        }
        return this.matched(true, context);
    };
    Shortcut.matchCode = function (code1, code2) {
        var _a, _b;
        return ((_a = code1 === null || code1 === void 0 ? void 0 : code1.toLocaleLowerCase) === null || _a === void 0 ? void 0 : _a.call(code1)) === ((_b = code2 === null || code2 === void 0 ? void 0 : code2.toLocaleLowerCase) === null || _b === void 0 ? void 0 : _b.call(code2));
    };
    Shortcut.sortCodes = function (codes) {
        return codes.map(function (code) { return code.toLocaleLowerCase(); }).sort();
    };
    return Shortcut;
}());
export { Shortcut };
