import { Operation } from './Operation';
import { IDesignerProps, IControlNodeMetaType } from '../types';
export interface ITreeNode {
    componentName?: string;
    operation?: Operation;
    hidden?: boolean;
    id?: string;
    props?: {
        [key: string]: any;
    };
    children?: ITreeNode[];
}
export interface INodeFinder {
    (node: TreeNode): boolean;
}
export declare class TreeNode {
    parent: TreeNode;
    root: TreeNode;
    operation: Operation;
    id: string;
    depth: number;
    hidden: boolean;
    componentName: string;
    props: ITreeNode['props'];
    children: TreeNode[];
    originDesignerProps: IDesignerProps;
    constructor(node?: ITreeNode, parent?: TreeNode);
    makeObservable(): void;
    set designerProps(props: IDesignerProps);
    get designerProps(): IDesignerProps;
    get previous(): TreeNode;
    get after(): TreeNode;
    get siblings(): TreeNode[];
    get index(): number;
    get childrens(): TreeNode[];
    getPrevious(step?: number): TreeNode;
    getAfter(step?: number): TreeNode;
    getSibling(index?: number): TreeNode;
    isMyAncestor(node: TreeNode): boolean;
    isMyParent(node: TreeNode): boolean;
    isMyParents(node: TreeNode): boolean;
    isMyChild(node: TreeNode): boolean;
    isMyChildren(node: TreeNode): boolean;
    triggerMutation(event: any): void;
    find(finder: INodeFinder): TreeNode;
    findAll(finder: INodeFinder): TreeNode[];
    distanceTo(node: TreeNode): number;
    crossSiblings(node: TreeNode): TreeNode[];
    matchNodeMeta(meta: IControlNodeMetaType): boolean;
    allowSibling(nodes: TreeNode[]): boolean;
    allowAppend(nodes: TreeNode[]): boolean;
    findById(id: string): TreeNode;
    getParents(node?: TreeNode): TreeNode[];
    getParentByDepth(depth?: number): any;
    contains(...nodes: TreeNode[]): boolean;
    eachChildren(callback?: (node: TreeNode) => void | boolean): void;
    resetNodesParent(nodes: TreeNode[], parent: TreeNode): TreeNode[];
    setNodeProps(props?: any): void;
    setComponentName(name: string): void;
    prependNode(...nodes: TreeNode[]): TreeNode[];
    appendNode(...nodes: TreeNode[]): TreeNode[];
    wrapNode(wrapper: TreeNode): TreeNode;
    insertAfter(...nodes: TreeNode[]): TreeNode[];
    insertBefore(...nodes: TreeNode[]): TreeNode[];
    insertChildren(start: number, ...nodes: TreeNode[]): TreeNode[];
    setNodeChildren(...nodes: TreeNode[]): TreeNode[];
    remove(): void;
    clone(parent?: TreeNode): any;
    from(node?: ITreeNode): void;
    serialize(): ITreeNode;
    create(node: ITreeNode, parent?: TreeNode): TreeNode;
}
