import { Viewport } from './Viewport';
import { Operation } from './Operation';
import { History } from './History';
import { uid } from '@designable/shared';
import { HistoryRedoEvent, HistoryUndoEvent } from '../events';
var Workspace = (function () {
    function Workspace(engine, props) {
        var _this = this;
        this.engine = engine;
        this.props = props;
        this.id = props.id || uid();
        this.title = props.title;
        this.description = props.description;
        this.viewport = new Viewport({
            engine: this.engine,
            workspace: this,
            viewportElement: props.viewportElement,
            contentWindow: props.contentWindow,
            nodeIdAttrName: this.engine.props.nodeIdAttrName,
        });
        this.outline = new Viewport({
            engine: this.engine,
            workspace: this,
            viewportElement: props.viewportElement,
            contentWindow: props.contentWindow,
            nodeIdAttrName: this.engine.props.outlineNodeIdAttrName,
        });
        this.operation = new Operation(this);
        this.history = new History(this, {
            onRedo: function (item) {
                _this.operation.hover.clear();
                _this.operation.dispatch(new HistoryRedoEvent(item));
            },
            onUndo: function (item) {
                _this.operation.hover.clear();
                _this.operation.dispatch(new HistoryUndoEvent(item));
            },
        });
    }
    Workspace.prototype.getEventContext = function () {
        return {
            workbench: this.engine.workbench,
            workspace: this,
            engine: this.engine,
            viewport: this.viewport,
        };
    };
    Workspace.prototype.attachEvents = function (container, contentWindow) {
        this.engine.attachEvents(container, contentWindow, this.getEventContext());
    };
    Workspace.prototype.detachEvents = function (container) {
        this.engine.detachEvents(container);
    };
    Workspace.prototype.dispatch = function (event) {
        this.engine.dispatch(event, this.getEventContext());
    };
    Workspace.prototype.serialize = function () {
        return {
            operation: this.operation.serialize(),
        };
    };
    Workspace.prototype.from = function (workspace) {
        if (workspace === null || workspace === void 0 ? void 0 : workspace.operation) {
            this.operation.from(workspace.operation);
        }
    };
    return Workspace;
}());
export { Workspace };
