"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Workspace = void 0;
var Viewport_1 = require("./Viewport");
var Operation_1 = require("./Operation");
var History_1 = require("./History");
var shared_1 = require("@designable/shared");
var events_1 = require("../events");
var Workspace = (function () {
    function Workspace(engine, props) {
        var _this = this;
        this.engine = engine;
        this.props = props;
        this.id = props.id || shared_1.uid();
        this.title = props.title;
        this.description = props.description;
        this.viewport = new Viewport_1.Viewport({
            engine: this.engine,
            workspace: this,
            viewportElement: props.viewportElement,
            contentWindow: props.contentWindow,
            nodeIdAttrName: this.engine.props.nodeIdAttrName,
        });
        this.outline = new Viewport_1.Viewport({
            engine: this.engine,
            workspace: this,
            viewportElement: props.viewportElement,
            contentWindow: props.contentWindow,
            nodeIdAttrName: this.engine.props.outlineNodeIdAttrName,
        });
        this.operation = new Operation_1.Operation(this);
        this.history = new History_1.History(this, {
            onRedo: function (item) {
                _this.operation.hover.clear();
                _this.operation.dispatch(new events_1.HistoryRedoEvent(item));
            },
            onUndo: function (item) {
                _this.operation.hover.clear();
                _this.operation.dispatch(new events_1.HistoryUndoEvent(item));
            },
        });
    }
    Workspace.prototype.getEventContext = function () {
        return {
            workbench: this.engine.workbench,
            workspace: this,
            engine: this.engine,
            viewport: this.viewport,
        };
    };
    Workspace.prototype.attachEvents = function (container, contentWindow) {
        this.engine.attachEvents(container, contentWindow, this.getEventContext());
    };
    Workspace.prototype.detachEvents = function (container) {
        this.engine.detachEvents(container);
    };
    Workspace.prototype.dispatch = function (event) {
        this.engine.dispatch(event, this.getEventContext());
    };
    Workspace.prototype.serialize = function () {
        return {
            operation: this.operation.serialize(),
        };
    };
    Workspace.prototype.from = function (workspace) {
        if (workspace === null || workspace === void 0 ? void 0 : workspace.operation) {
            this.operation.from(workspace.operation);
        }
    };
    return Workspace;
}());
exports.Workspace = Workspace;
