"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GlobalRegistry = void 0;
var shared_1 = require("@designable/shared");
var path_1 = require("@formily/path");
var reactive_1 = require("@formily/reactive");
var getBrowserlanguage = function () {
    var _a;
    if (!window.navigator) {
        return 'en';
    }
    return (window.navigator['browserlanguage'] || ((_a = window.navigator) === null || _a === void 0 ? void 0 : _a.language) || 'en');
};
var getISOCode = function (language) {
    var isoCode = DESINGER_LOCALES.language;
    if (DESINGER_LOCALES.messages[language]) {
        return language;
    }
    shared_1.each(DESINGER_LOCALES.messages, function (_, key) {
        if (key.indexOf(language) > -1 || String(language).indexOf(key) > -1) {
            isoCode = key;
            return false;
        }
    });
    return isoCode;
};
var DESINGER_PROPS_MAP = {
    Root: {
        droppable: true,
        cloneable: false,
        deletable: false,
    },
};
var DESINGER_ICONS_MAP = {};
var DESINGER_LOCALES = reactive_1.define({
    messages: {},
    language: getBrowserlanguage(),
}, {
    language: reactive_1.observable.ref,
});
var mergeLocales = function (target, source) {
    if (shared_1.isPlainObj(target) && shared_1.isPlainObj(source)) {
        shared_1.each(source, function (value, key) {
            target[key] = mergeLocales(target[key], value);
        });
        return target;
    }
    return source;
};
var DESIGNER_GlobalRegistry = {
    setComponentDesignerProps: function (componentName, props) {
        var originProps = exports.GlobalRegistry.getComponentDesignerProps(componentName);
        DESINGER_PROPS_MAP[componentName] = function (node) {
            if (shared_1.isFn(originProps)) {
                if (shared_1.isFn(props)) {
                    return __assign(__assign({}, originProps(node)), props(node));
                }
                else {
                    return __assign(__assign({}, originProps(node)), props);
                }
            }
            else if (shared_1.isFn(props)) {
                return __assign(__assign({}, originProps), props(node));
            }
            else {
                return __assign(__assign({}, originProps), props);
            }
        };
    },
    getComponentDesignerProps: function (componentName) {
        return DESINGER_PROPS_MAP[componentName] || {};
    },
    registerDesignerProps: function (map) {
        shared_1.each(map, function (props, componentName) {
            exports.GlobalRegistry.setComponentDesignerProps(componentName, props);
        });
    },
    registerDesignerIcons: function (map) {
        Object.assign(DESINGER_ICONS_MAP, map);
    },
    getDesignerIcon: function (name) {
        return DESINGER_ICONS_MAP[name];
    },
    setDesignerLanguage: function (lang) {
        DESINGER_LOCALES.language = lang;
    },
    getDesignerLanguage: function () {
        return DESINGER_LOCALES.language;
    },
    getDesignerMessage: function (token) {
        var lang = getISOCode(DESINGER_LOCALES.language);
        var locale = DESINGER_LOCALES.messages[lang];
        if (!locale) {
            for (var key in DESINGER_LOCALES.messages) {
                var message = path_1.Path.getIn(DESINGER_LOCALES.messages[key], token);
                if (message)
                    return message;
            }
            return;
        }
        return path_1.Path.getIn(locale, token);
    },
    registerDesignerLocales: function () {
        var packages = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            packages[_i] = arguments[_i];
        }
        packages.forEach(function (locales) {
            mergeLocales(DESINGER_LOCALES.messages, locales);
        });
    },
};
exports.GlobalRegistry = window['__DESIGNER_GlobalRegistry__'] || DESIGNER_GlobalRegistry;
